﻿namespace Pikta.Editor.Actions
{
    using System;
    using System.Collections.Generic;
    using System.Diagnostics.Contracts;
    using System.Linq;
    using System.Text;

    public class AddElementAction : HistoryAction
    {
        public AddElementAction(Scene scene, Element element, string name = null, Element container = null)
        {
            Contract.Requires(scene != null);
            Contract.Requires(element != null);
            Contract.Requires(container == null || container.IsContainer());

            if (container == null)
            {
                // try prototype's container; otherwise, use root
                container = scene.GetParentContainer(element.Prototype) ?? scene.Root;
            }

            base.Name = string.Format("Přidání elementu {0} do {1}", element, container);

            if (!string.IsNullOrEmpty(name) && Namespace.Global.Contains(name))
            {
                base.DisableReason = string.Format("Element pojmenovaný \"{0}\" již existuje.", name);
                return;
            }

            // neither container nor its clones cannot be added below itself
            Element parentElement = container.GetElement(KnownNames.ContentProperty);
            var conflictingContainer = scene.GetSuperElementsOf(parentElement)
                .Concat(new[] { parentElement })
                .FirstOrDefault(c => c == element || element.IsDescendantOf(c));
            if (conflictingContainer != null)
            {
                base.DisableReason = string.Format("Element \"{0}\" nelze přidat pod kontejner \"{1}\". Došlo by k vytvoření cyklu.",
                    string.IsNullOrEmpty(name) ? element.ToString() : name,
                    conflictingContainer);
                return;
            }
            
            base.PerformAction = () =>
                {
                    if (string.IsNullOrEmpty(name))
                    {
                        name = Namespace.Global.AddWithUniqueName(element);
                    }
                    else
                    {
                        Namespace.Global[name] = element;
                    }

                    container.GetContainerChildren().Add(element);

                    // update the name for we know it by now
                    base.Name = string.Format("Přidání elementu {0} do {1}", name, container);
                };
            base.UndoAction = () =>
                {
                    Namespace.Global.Remove(element);
                    container.GetContainerChildren().Remove(element);
                };
        }
    }
}