﻿namespace Pikta.Editor.Actions
{
    using System;
    using System.Collections.Generic;
    using System.Diagnostics.Contracts;
    using System.Linq;
    using System.Text;
    using Pikta.Expressions;

    public class GenerateGridAction : HistoryAction
    {
        public GenerateGridAction(Scene scene, Element targetContainer, string name, Element gridTemplate, int rowCount, int columnCount,
            Element itemTemplate, Func<Element, Element> clone)
        {
            Contract.Requires(gridTemplate != null);
            Contract.Requires(rowCount > 0);
            Contract.Requires(columnCount > 0);
            Contract.Requires(itemTemplate != null);

            base.Name = "Generování mřížky " + (name ?? string.Empty);

            // create temporary generator
            var g = clone(Namespace.Global[KnownNames.Generator]);
            g["počet"].Value = new Constant(rowCount * columnCount);
            g["kontejner"].Value = new Constant(gridTemplate);
            g["šablona"].Value = new Constant(itemTemplate);

            var runAction = new RunGeneratorAction(scene, g, clone);

            base.DisableReason = runAction.DisableReason;

            if (!this.CanExecute)
            {
                return;
            }

            runAction.PerformAction();

            var resultGrid = g.GetElement("výsledek");
            resultGrid["sloupce"].Value = new Constant(columnCount);
            resultGrid["řádky"].Value = new Constant(rowCount);

            var addResultAction = new AddElementAction(scene, resultGrid, name, targetContainer);

            if (!addResultAction.CanExecute)
            {
                base.DisableReason = addResultAction.DisableReason;
                return;
            }

            Property itemGridProperty = itemTemplate["mřížka"];
            Property itemIProperty = itemTemplate["i"];
            Property itemColumnProperty = itemTemplate["sloupec"];
            Property itemRowProperty = itemTemplate["řádek"];

            // capture current item property states
            var itemPropertyStates =
                new[] { itemGridProperty, itemIProperty, itemColumnProperty, itemRowProperty }
                .Select(p => p.CaptureState()).ToArray();

            base.PerformAction = () =>
                {
                    var dotI = new SymbolExpression(null, "i");
                    var gr = new SymbolExpression(null, "mřížka", "sloupce");

                    addResultAction.PerformAction();

                    // TODO: Tyto modifikace šablony nejsou dobré. Jak se jim vyhnout lépenež "meziklonem" šablony?

                    // add specific properties
                    itemGridProperty.Value = new SymbolExpression(resultGrid.GetName());
                    itemIProperty.Value = new Constant(0);
                    itemColumnProperty.Value = new Modulo(dotI, gr);
                    itemRowProperty.Value = new FunctionCall(new FunctionDefinition("Celé", 1), new[] { new Division(dotI, gr) });
                };

            base.UndoAction = () =>
                {
                    Array.ForEach(itemPropertyStates, s => s.Restore());

                    addResultAction.UndoAction();
                };
        }
    }
}
