﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace Pikta.Editor.Actions
{
    public class MultiAction : HistoryAction
    {
        public MultiAction(string name, IEnumerable<HistoryAction> actions)
        {
            // ensure the enumeration is materialized so actions' constructors are called now
            var materializedActions = actions.ToArray();

            base.Name = name;
            base.DisableReason = materializedActions.Select(a => a.DisableReason).FirstOrDefault(r => !string.IsNullOrEmpty(r));

            base.PerformAction = () =>
                {
                    foreach (var a in materializedActions)
                    {
                        a.PerformAction();
                    }
                };

            base.UndoAction = () =>
                {
                    foreach (var a in materializedActions.Reverse())
                    {
                        a.UndoAction();
                    }
                };
        }
    }
}
