﻿namespace Pikta.Editor.Actions
{
    using System;
    using System.Collections.Generic;
    using System.Diagnostics.Contracts;
    using System.Linq;
    using System.Text;

    public class RemoveElementAction : HistoryAction
    {
        public RemoveElementAction(Scene scene, Element element)
        {
            Contract.Requires(scene != null);
            Contract.Requires(element != null);

            string elementName = element.GetName();

            base.Name = string.Format("Odebrání elementu {0}.", element);

            if (element.GetName() == KnownNames.Root)
            {
                base.DisableReason = "Kořenový element nelze odebrat.";
                return;
            }

            // find the containers
            var childrenCollections = (from c in scene.AllSceneContainers
                                       let children = c.GetContainerChildren()
                                       where children.Contains(element)
                                       select children).ToArray();
            var childIndices = new List<int>(childrenCollections.Length);

            base.PerformAction = () =>
                {
                    // remove element from containers
                    foreach (IList<Element> collection in childrenCollections)
                    {
                        int index = collection.IndexOf(element);
                        childIndices.Add(index);
                        collection.RemoveAt(index);
                    }

                    // remove element from the namespace
                    Namespace.Global.Remove(element);
                };

            base.UndoAction = () =>
                {
                    // add element to the namespace
                    if (elementName != null)
                    {
                        Namespace.Global[elementName] = element;
                    }

                    // insert element to original containers
                    for (int i = 0; i < childrenCollections.Length; i++)
                    {
                        childrenCollections[i].Insert(childIndices[i], element);
                    }
                };
        }
    }
}
