﻿namespace Pikta.Editor.Actions
{
    using System;
    using System.Collections.Generic;
    using System.Diagnostics.Contracts;
    using System.Linq;
    using System.Text;
    using Pikta.Expressions;

    public class RenameElementAction : HistoryAction
    {
        public RenameElementAction(Scene scene, Element element, string newName)
        {
            Contract.Requires(element != null);
            Contract.Requires(string.IsNullOrEmpty(newName) || Parser.IsValidElementName(newName));

            base.Name = string.Format("Přejmenování elementu {0} na {1}", element, newName);

            if (element.GetName() == KnownNames.Root)
            {
                base.DisableReason = "Kořenový element nelze přejmovat.";
                return;
            }

            if (Namespace.Global.Contains(newName))
            {
                base.DisableReason = string.Format("Element pojmenovaný \"{0}\" již existuje.", newName);
                return;
            }

            string oldName = Namespace.Global.GetName(element);

            base.PerformAction = () =>
                {
                    if (oldName != null)
                    {
                        if (string.IsNullOrEmpty(newName))
                        {
                            // remove old name
                            Namespace.Global.Remove(element);
                        }
                        else
                        {
                            // rename element
                            Namespace.Global.Rename(oldName, newName);
                            // rename all references
                            RenameAllReferences(scene.AllElements, oldName, newName);
                        }
                    }
                    else if(!string.IsNullOrEmpty(newName))
                    {
                        // assign new name
                        Namespace.Global[newName] = element;
                    }
                };

            base.UndoAction = () =>
                {
                    if (oldName == null && !string.IsNullOrEmpty(newName))
                    {
                        // remove new name
                        Namespace.Global.Remove(element);
                    }
                    else
                    {
                        if (string.IsNullOrEmpty(newName))
                        {
                            // remove old name
                            Namespace.Global.Remove(element);
                        }
                        else
                        {
                            // rename all references
                            RenameAllReferences(scene.AllElements, newName, oldName);
                            // rename element
                            Namespace.Global.Rename(newName, oldName);
                        }
                    }
                };
        }

        private void RenameAllReferences(IEnumerable<Element> allElements, string oldName, string newName)
        {
            foreach (var e in allElements)
            {
                foreach (var def in e.LocalPropertyDefinitions)
                {
                    if (def.Value.AllSymbolExpressions.Any(se => NameComparer.Instance.Equals(oldName, se.ElementName)))
                    {
                        // TODO: Transformovat vyraz def.Value
                    }
                }
            }
        }
    }
}
