﻿namespace Pikta.Editor.Actions
{
    using System;
    using System.Collections.Generic;
    using System.Diagnostics.Contracts;
    using System.Linq;
    using System.Text;
    using Pikta.Expressions;

    public class RunGeneratorAction : HistoryAction
    {
        public RunGeneratorAction(Scene scene, Element generator, Func<Element, Element> clone)
        {
            Contract.Requires(scene != null);
            Contract.Requires(generator != null);
            Contract.Requires(clone != null);

            base.Name = "Spuštění generátoru " + generator;

            // collect parameters
            int count = generator.GetIntNumber("počet");
            Element containerTemplate = generator.GetElement("kontejner");
            Element itemTemplate = generator.GetElement("šablona");
            Expression previousResult = generator["výsledek"].Value;

            if (containerTemplate == null)
            {
                base.DisableReason = "Generátor nemá zadaný cílový kontejner.";
                return;
            }
            if (itemTemplate == null)
            {
                base.DisableReason = "Generátor nemá zadanou šablonu.";
                return;
            }

            // clone container
            Element result = clone(containerTemplate);
            var children = result.GetContainerChildren();

            // clone child items
            for (int i = 0; i < count; i++)
            {
                Element item = clone(itemTemplate);

                item["i"].Value = new Constant(i);
                children.Add(item);
            }

            Expression newResult = new Constant(result);

            Element generatorContainer = scene.GetParentContainer(generator);
            var addResultAction = generatorContainer != null ? new AddElementAction(scene, result, null, generatorContainer) : null;

            base.PerformAction =
                () =>
                {
                    if (addResultAction != null)
                    {
                        addResultAction.PerformAction();
                    }
                    else
                    {
                        generator["výsledek"].Value = newResult;
                    }
                };
            base.UndoAction =
                () =>
                {
                    if (addResultAction != null)
                    {
                        addResultAction.UndoAction();
                    }
                    else
                    {
                        generator["výsledek"].Value = previousResult;
                    }
                };
        }
    }
}
