﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Pikta.Expressions;

namespace Pikta.Editor.Actions
{
    public class SetPropertyReference : HistoryAction
    {
        public SetPropertyReference(Property propertyToSet, Property referenced)
        {
            Expression expression;

            base.Name = string.Format("Nastavení odkazu na {0} do {1}", referenced, propertyToSet);

            if (referenced.Element == propertyToSet.Element || propertyToSet.Element.IsDescendantOf(referenced.Element))
            {
                // relative reference
                expression = new SymbolExpression(null, referenced.Name);
            }
            else
            {
                // absolute reference
                string referencedElementName = referenced.Element.GetName();

                if (referencedElementName != null)
                {
                    expression = new SymbolExpression(referencedElementName, referenced.Name);
                }
                else
                {
                    base.DisableReason = string.Format("Na element vlastnosti \"{0}\" se nelze odkázat, protože nemá název.", referenced.Name);
                    return;
                }
            }

            var changeAction = new ChangePropertyValueAction(propertyToSet, expression);

            base.DisableReason = changeAction.DisableReason;
            base.PerformAction = changeAction.PerformAction;
            base.UndoAction = changeAction.UndoAction;
        }
    }
}
