﻿namespace Pikta.Editor
{
    using System;
    using System.Collections.Generic;
    using System.Drawing;
    using System.Linq;
    using System.Text;

    public class BitmapVisual : Shape
    {
        private static readonly BitmapCache cache = new BitmapCache();

        public BitmapVisual()
            : base(KnownNames.Bitmap)
        {
        }

        public override void DrawElement(Graphics g, Element e, Rectangle container)
        {
            string source = GetSource(e);

            if (string.IsNullOrEmpty(source))
            {
                return;
            }

            Image image = cache[source];

            int width = GetWidth(e);
            int height = GetHeight(e);

            var r = new Rectangle(container.Location, new Size(width, height));

            OnRectangleChanged(e, r);

            if (image != null)
            {
                g.DrawImage(image, r);
            }
        }

        private string GetSource(Element e)
        {
            return e.GetString("zdroj");
        }

        internal static int GetBitmapWidth(string fileName)
        {
            var bmp = cache[fileName];
            return bmp != null ? bmp.Width : 1;
        }

        internal static int GetBitmapHeight(string fileName)
        {
            var bmp = cache[fileName];
            return bmp != null ? bmp.Height : 1;
        }

        public static void SetCurrentPath(string currentPath)
        {
            cache.SetCurrentPath(currentPath);
        }
    }
}
