﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Drawing;

namespace Pikta.Editor
{
	public class Canvas : ContainerVisual
	{
		public Canvas(IVisualProvider visualProvider)
			: base(KnownNames.Canvas, visualProvider)
		{
		}

		public override void DrawElement(Graphics g, Element e, Rectangle container)
		{
            // ziskame parametry
            var sirka = GetWidth(e);
            var vyska = GetHeight(e);

            var plocha = new Rectangle(container.Location, new Size(sirka, vyska));

            OnRectangleChanged(e, plocha);

            // vykreslime pozadi
            using (var vypln = GetFill(e))
            {
                g.FillRectangle(vypln, plocha);
            }

            // okraj
            using (var p = GetStroke(e))
            {
                g.DrawRectangle(p, plocha);
            }

            using (var vnitrek = new Region(plocha))
            {
                var state = ClipToRegion(g, vnitrek);

                // vykreslime obsah
                foreach (Element podelement in GetContent(e))
                {
                    var plochaProElement = plocha;
                    plochaProElement.Offset(GetX(podelement), GetY(podelement));

                    GetHandler(podelement).DrawElement(g, podelement, plochaProElement);
                }

                g.Restore(state);
            }
        }

        private int GetX(Element e) { return e.GetIntNumber("x", 0); }
    
        private int GetY(Element e) { return e.GetIntNumber("y", 0); }
    }
}
