﻿namespace Pikta.Editor
{
    using System;
    using System.Collections.Generic;
    using System.Linq;
    using System.Text;
    using System.Windows.Forms;
    using System.Drawing;
    using Pikta.Expressions;
    using Pikta.Editor.Actions;

    public class CanvasDraggingHandler : DraggingHandler
    {
        private IList<IPropertyState> originalX;
        private IList<IPropertyState> originalY;

        public CanvasDraggingHandler(Scene scene, View view, Element element)
            : base(scene, view, element)
        {
            IsMultiDrag = View.Selection.Selected.Count > 1;// && View.Selection.Selected.Contains(element);

            //if (IsMultiDrag)
            {
                Moved = View.Selection.Selected;
            }
            //else
            //{
            //    Moved = new[] { Element };
            //}
        }

        private IList<Element> Moved { get; set; }

        private Point LastPoint { get; set; }
        /// <summary>Hodnota určující, zda se jedná o tažení více vybraných elementů.</summary>
        private bool IsMultiDrag { get; set; }

        public override void Start(MouseEventArgs e)
        {
            base.Start(e);

            LastPoint = e.Location;

            this.originalX = Moved.Select(element => element[KnownNames.XProperty].CaptureState()).ToArray();
            this.originalY = Moved.Select(element => element[KnownNames.YProperty].CaptureState()).ToArray();
        }

        public override void Move(MouseEventArgs e)
        {
            base.Move(e);

            int dx = e.X - LastPoint.X;
            int dy = e.Y - LastPoint.Y;

            foreach (Element element in Moved)
            {
                MoveElement(element, dx, dy);
            }

            this.View.Redraw();

            LastPoint = e.Location;
        }

        private void MoveElement(Element element, int dx, int dy)
        {
            double x = element.GetNumber(KnownNames.XProperty, 0);
            double y = element.GetNumber(KnownNames.YProperty, 0);

            element[KnownNames.XProperty].Value = new Constant(x + dx);
            element[KnownNames.YProperty].Value = new Constant(y + dy);
        }

        public override void Finish(MouseEventArgs e)
        {
            base.Finish(e);

            if (!Moved.Any())
            {
                return;
            }

            View.ActionTarget.PerformAction(
                new MultiAction(
                    Moved.Count > 1 ? string.Format("Přesun {0:N0} elementů", Moved.Count) : ("Přesun elementu " + Moved.Single()),
                    Moved.SelectMany(
                        (element, i) =>
                        {
                            Property x = element[KnownNames.XProperty];
                            Property y = element[KnownNames.YProperty];

                            return new[]
                                {
                                    new ChangePropertyValueAction(x, new Constant(element.GetNumber(KnownNames.XProperty, 0)), this.originalX[i]),
                                    new ChangePropertyValueAction(y, new Constant(element.GetNumber(KnownNames.YProperty, 0)), this.originalY[i])
                                };
                        })));
        }
    }
}
