﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Pikta.Expressions;

namespace Pikta.Editor.Definitions
{
    public static class SceneFunctions
    {
        public static void Define(Scene scene)
        {
            // scene hierarchy
            Add("Kontejner", 1, (_, args) =>
                {
                    var e = Converters.ToElement(args.Single());
                    var k = scene.GetParentContainer(e);
                    if (k != null)
                    {
                        return Result.Success(k);
                    }
                    else
                    {
                        return Result.Error(string.Format("Element \"{0}\" nemá kontejner.", e));
                    }
                });
            Add("SoučástNaPozici", 2, (_, args) =>
                {
                    var kontejner = Converters.ToElement(args.ElementAt(0));
                    var index = (int)Converters.ToNumber(args.ElementAt(1));

                    var kolekce = kontejner.GetContainerChildren();

                    if (index < kolekce.Count)
                    {
                        return Result.Success(kolekce[index]);
                    }
                    else
                    {
                        return Result.Error("Pozice je mimo seznam součástí kontejneru.");
                    }
                });

            // text
            Add("ŠířkaTextu", 2, (_, args) =>
                {
                    string text = Converters.ToText(args.ElementAt(0));
                    double size = Converters.ToNumber(args.ElementAt(1));

                    return Result.Success(Text.GetTextWidth(text, size));
                });
            Add("VýškaTextu", 2, (_, args) =>
            {
                string text = Converters.ToText(args.ElementAt(0));
                double size = Converters.ToNumber(args.ElementAt(1));

                return Result.Success(Text.GetTextHeight(text, size));
            });

            // bitmapa
            Add("ŠířkaBitmapy", 1, (_, args) =>
                {
                    string fileName = Converters.ToText(args.Single());
                    if (!string.IsNullOrEmpty(fileName))
                        return Result.Success(BitmapVisual.GetBitmapWidth(fileName));
                    else
                        return Result.Success(0);
                });
            Add("VýškaBitmapy", 1, (_, args) =>
            {
                string fileName = Converters.ToText(args.Single());
                if (!string.IsNullOrEmpty(fileName))
                    return Result.Success(BitmapVisual.GetBitmapHeight(fileName));
                else
                    return Result.Success(0);
            });
        }

        private static void Add(string name, int arity, Evaluation.FunctionBody body)
        {
            Evaluation.FunctionsLibrary[new FunctionDefinition(name, arity)] = body;
        }
    }
}
