﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Globalization;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using Pikta.Expressions;

namespace Pikta.Editor
{
    public partial class ExpressionEditorForm : Form
    {
        private Property property;

        public ExpressionEditorForm()
        {
            InitializeComponent();
        }

        public Property Property
        {
            get { return property; }
            set
            {
                if (property != value)
                {
                    property = value;

                    if (Property != null)
                    {
                        if (Property.Definition != null)
                        {
                            this.Text = string.Format("Editace vlastnosti {0}", Property.Name);
                            this.nameComboBox.Text = Property.Name;
                            this.nameComboBox.Enabled = false;
                            this.valueTextBox.Text = Property.Value.ToString();
                            NewPropertyName = Property.Name;
                        }
                        else
                        {
                            this.Text = "Přidat vlastnost";
                            this.nameComboBox.Enabled = true;
                            FillOverridableNames();
                            this.valueTextBox.Clear();
                        }
                    }
                }
            }
        }

        public string NewPropertyName { get; set; }

        public Expression NewPropertyValue { get; set; }

        private void valueTextBox_TextChanged(object sender, EventArgs e)
        {
            this.okButton.Enabled = !string.IsNullOrWhiteSpace(valueTextBox.Text);
        }

        private void okButton_Click(object sender, EventArgs e)
        {
            NewPropertyName = nameComboBox.Text.Trim();

            if (!Parser.IsValidPropertyName(NewPropertyName))
            {
                Program.ShowInfo("Zadaný název vlastnosti není platný.");
                nameComboBox.Select();
                return;
            }

            try
            {
                NewPropertyValue = Parser.Parse(valueTextBox.Text, CultureInfo.CurrentCulture);
            }
            catch (Exception ex)
            {
                Program.ShowError(ex.Message);
                valueTextBox.Select();
                return;
            }

            var pokusneVyhodnoceni = Property.Element.Evaluate(NewPropertyValue);

            if (pokusneVyhodnoceni.HasError)
            {
                Program.ShowError("Zadaný výraz se nepodařilo vyhodnotit." + Environment.NewLine + Environment.NewLine + pokusneVyhodnoceni.ErrorMessage);
                valueTextBox.Select();
                return;
            }

            DialogResult = DialogResult.OK;
        }

        private void FillOverridableNames()
        {
            nameComboBox.Items.Clear();

            var ancestorsProperties =
                Property.Element.Prototype != null ?
                Property.Element.Prototype.PropertyDefinitions :
                Enumerable.Empty<PropertyDefinition>();

            var notOverriden = from def in ancestorsProperties
                               where !Property.Element.DefinesDirectly(def.Name)
                               select def;

            nameComboBox.Items.AddRange((from def in notOverriden
                                         select def.Name).ToArray());
        }
    }
}
