﻿namespace Pikta.Editor
{
    using System;
    using System.Collections.Generic;
    using System.Drawing;
    using System.Linq;
    using System.Text;

    public class Grid : ContainerVisual
    {
        public Grid(IVisualProvider provider)
            : base(KnownNames.Grid, provider)
        {
        }

        public override void DrawElement(Graphics g, Element e, Rectangle container)
        {
            int sirka = GetWidth(e);
            int vyska = GetHeight(e);

            var r = new Rectangle(container.X, container.Y, sirka, vyska);

            OnRectangleChanged(e, r);

            // background
            using (var stetec = GetFill(e))
            {
                g.FillRectangle(stetec, r);
                //LastRectangles[element] = new Rectangle(rect.X, rect.Y, sirka, vyska);
            }

            int pocetSloupcu = GetColumnsCount(e);
            int pocetRadku = GetRowsCount(e);

            float sirkaSloupce = (float)sirka / pocetSloupcu;
            float vyskaRadku = (float)vyska / pocetRadku;

            // gridlines
            using (var p = GetStroke(e))
            {
                for (int i = 0; i <= pocetSloupcu; i++)
                {
                    int x = (int)Math.Round(r.X + i * sirkaSloupce);
                    g.DrawLine(p, x, r.Top, x, r.Bottom);
                }
                for (int j = 0; j <= pocetRadku; j++)
                {
                    int y = (int)Math.Round(r.Y + j * vyskaRadku);
                    g.DrawLine(p, r.Left, y, r.Right, y);
                }
            }

            // content
            using (var vnitrek = new Region(r))
            {
                var state = ClipToRegion(g, vnitrek);

                foreach (Element potomek in e.GetContainerChildren())
                {
                    int okraj = 0;// potomek.GetInt("margin");
                    int pw = GetWidth(potomek);
                    int ph = GetHeight(potomek); ;

                    int sloupec = GetColumn(potomek);
                    int radek = GetRow(potomek);

                    sloupec = Math.Max(0, Math.Min(sloupec, pocetSloupcu - 1));
                    radek = Math.Max(0, Math.Min(radek, pocetRadku - 1));

                    int bunkaX = (int)Math.Round(sloupec * sirkaSloupce);
                    int bunkaY = (int)Math.Round(radek * vyskaRadku);

                    int x = ApplyHorizontalAlignment(potomek, bunkaX, (int)sirkaSloupce, pw, okraj);
                    int y = ApplyVerticalAlignment(potomek, bunkaY, (int)vyskaRadku, ph, okraj);

                    base.GetHandler(potomek).DrawElement(g, potomek, new Rectangle(container.X + x, container.Y + y, (int)sirkaSloupce, (int)vyskaRadku));
                }

                g.Restore(state);
            }
        }

        private int GetRow(Element e)
        {
            return e.GetIntNumber("řádek", 0);
        }

        private int GetColumn(Element e)
        {
            return e.GetIntNumber("sloupec", 0);
        }

        private int GetRowsCount(Element e)
        {
            return Math.Max(1, e.GetIntNumber("řádky"));
        }

        private int GetColumnsCount(Element e)
        {
            return Math.Max(1, e.GetIntNumber("sloupce"));
        }
    }
}
