﻿namespace Pikta.Editor
{
    using System;
    using System.IO;
    using System.Text;

    /// <summary>
    /// Helper class for obtaining information from EXE files.
    /// </summary>
    internal static class PEHelper
    {
        private static DateTime TimeStampBase = new DateTime(1970, 1, 1, 0, 0, 0, DateTimeKind.Utc);

        /// <summary>
        /// Returns the specified PE file UTC time stamp.
        /// </summary>
        /// <param name="fileName">The PE file name. Cannot be null.</param>
        /// <exception cref="System.ArgumentException">Invalid file name.</exception>
        /// <exception cref="System.IO.IOException">Cannot find the time stamp.</exception>
        public static DateTime GetTimeStamp(string fileName)
        {
            var soubor = new FileStream(fileName, FileMode.Open, FileAccess.Read);
            using (var ctenar = new BinaryReader(soubor, Encoding.ASCII))
            {
                // presunem se na offset PE hlavicky
                soubor.Position = 0x3C;
                // precteme ho
                var offset_pe = ctenar.ReadUInt32();
                // presunem se tam
                soubor.Position = (long)offset_pe;

                // posunem se na pozici casoveho razitka
                soubor.Position += 8;
                // precteme razitko
                long razitko = ctenar.ReadUInt32();

                return TimeStampBase.AddSeconds(razitko);
            }
        }
    }
}
