﻿namespace Pikta.Editor
{
    using System;
    using System.Collections.Generic;
    using System.Diagnostics.Contracts;
    using System.Linq;
    using System.Text;
    using System.Windows.Forms;
    using Pikta.Editor.Actions;
    using Pikta.Expressions;

    public class PropertyEditor
    {
        public HistoryAction AddNewProperty(Element element)
        {
            using (var editor = new ExpressionEditorForm())
            {
                editor.Property = new Property(element, string.Empty);

                if (editor.ShowDialog() == DialogResult.OK)
                {
                    string name = editor.NewPropertyName;
                    Expression newValue = editor.NewPropertyValue;
                    var property = new Property(element, name);

                    return new AddPropertyAction(property, newValue);
                }
                else
                {
                    return null;
                }
            }
        }

        [Pure]
        public bool HasColorValue(Property property)
        {
            if (property == null) return false;
            Element element = property.Element.GetElement(property.Name);
            return (element != null && element.IsDescendantOf(KnownNames.Color));
        }

        public HistoryAction EditColor(Property property)
        {
            Contract.Requires(property != null);
            Contract.Requires(HasColorValue(property));

            using (var dialog = new ColorDialog())
            {
                dialog.Color = property.Element.GetColor(property.Name);

                if (dialog.ShowDialog() == DialogResult.OK)
                {
                    Expression newValue = new Constant(dialog.Color.ToElement());

                    return new ChangePropertyValueAction(property, newValue);

                }
                else
                {
                    return null;
                }
            }
        }

        public HistoryAction EditExpression(Property property)
        {
            Contract.Requires(property != null);

            using (var editor = new ExpressionEditorForm())
            {
                editor.Property = property;

                if (editor.ShowDialog() == DialogResult.OK)
                {
                    Expression newValue = editor.NewPropertyValue;

                    return new ChangePropertyValueAction(property, newValue);
                }
                else
                {
                    return null;
                }
            }
        }
    }
}
