﻿using System;
using System.Collections.Generic;
using System.Drawing;
using System.Linq;
using System.Text;

namespace Pikta.Editor
{
    public abstract class Shape : Visual
    {
        protected Shape(string prototypeName)
            : base(prototypeName)
        {
        }
    
        public int GetWidth(Element e) { return e.GetIntNumber("šířka", 10); }

        public int GetHeight(Element e) { return e.GetIntNumber("výška", 10); }

        protected float GetThickness(Element e) { return e.GetFloatNumber("tloušťka"); }

        protected Pen GetStroke(Element e) { return new Pen(e.GetColor("okraj"), GetThickness(e)); }

        protected Brush GetFill(Element e) { return new SolidBrush(e.GetColor("výplň")); }

        protected static HorizontalPosition GetHorizontalPosition(Element e)
        {
            var poloha = e.GetElement("horizontálníPoloha");

            var left = Namespace.Global["Vlevo"];
            var center = Namespace.Global["NaStřed"];
            var right = Namespace.Global["Vpravo"];
            var stretch = Namespace.Global["Roztažení"];

            if (poloha == left)
            {
                return HorizontalPosition.Left;
            }
            else if (poloha == center)
            {
                return HorizontalPosition.Center;
            }
            else if(poloha == right)
            {
                return HorizontalPosition.Right;
            }
            else
            {
                return HorizontalPosition.Stretch;
            }
        }

        protected static VerticalPosition GetVerticalPosition(Element e)
        {
            var poloha = e.GetElement("vertikálníPoloha");

            var top = Namespace.Global["Nahoru"];
            var center = Namespace.Global["NaStřed"];
            var bottom = Namespace.Global["Dolů"];
            var stretch = Namespace.Global["Roztažení"];

            if (poloha == top)
            {
                return VerticalPosition.Top;
            }
            else if (poloha == center)
            {
                return VerticalPosition.Center;
            }
            else if (poloha == bottom)
            {
                return VerticalPosition.Bottom;
            }
            else
            {
                return VerticalPosition.Stretch;
            }
        }

        protected static int ApplyHorizontalAlignment(Element element, int x, int cellWidth, int elementWidth, int margin)
        {
            switch (GetHorizontalPosition(element))
            {
                case HorizontalPosition.Center:
                    return x + (cellWidth - elementWidth) / 2;
                case HorizontalPosition.Right:
                    return x + cellWidth - elementWidth - margin - 1;
                case HorizontalPosition.Stretch:
                // TODO: Roztazeni
                case HorizontalPosition.Left:
                default:
                    return x + margin;
            }
        }

        protected static int ApplyVerticalAlignment(Element element, int y, int cellHeight, int elementHeight, int margin)
        {
            switch (GetVerticalPosition(element))
            {
                case VerticalPosition.Center:
                    return y + (cellHeight - elementHeight) / 2;
                case VerticalPosition.Bottom:
                    return y + cellHeight - elementHeight - margin - 1;
                case VerticalPosition.Top:
                case VerticalPosition.Stretch:
                default:
                    return y + margin;
            }
        }
    }

    public enum HorizontalPosition
    {
        Left,
        Center,
        Right,
        Stretch
    }

    public enum VerticalPosition
    {
        Top,
        Center,
        Bottom,
        Stretch
    }
}
