﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Drawing;
using System.Windows.Forms;
using Pikta.Expressions;
using Pikta.Editor.Actions;

namespace Pikta.Editor
{
    class SizeDraggingHandler : DraggingHandler
    {
        private IList<IPropertyState> originalWidth;
        private IList<IPropertyState> originalHeight;

        public SizeDraggingHandler(Scene scene, View view, SizeGripKind kind)
            : base(scene, view, null)
        {
            Kind = kind;
        }

        private IList<Element> Sized { get { return View.Selection.Selected; } }
        private SizeGripKind Kind { get; set; }
        private Point LastPoint { get; set; }

        public override void Start(MouseEventArgs e)
        {
            base.Start(e);

            switch (Kind)
            {
                case SizeGripKind.Width: View.Cursor = Cursors.SizeWE; break;
                case SizeGripKind.Height: View.Cursor = Cursors.SizeNS; break;
                case SizeGripKind.WidthHeight: View.Cursor = Cursors.SizeNWSE; break;
                default:
                    break;
            }

            this.originalWidth = Sized.Select(element => element[KnownNames.WidthProperty].CaptureState()).ToArray();
            this.originalHeight = Sized.Select(element => element[KnownNames.HeightProperty].CaptureState()).ToArray();

            LastPoint = e.Location;
        }

        public override void Move(MouseEventArgs e)
        {
            base.Move(e);

            int dx = e.X - LastPoint.X;
            int dy = e.Y - LastPoint.Y;

            foreach (Element vybrany in Sized)
            {
                SizeElement(vybrany, dx, dy);
            }

            LastPoint = e.Location;
        }

        private void SizeElement(Element element, int dx, int dy)
        {
            double x = element.GetNumber(KnownNames.WidthProperty, 0);
            double y = element.GetNumber(KnownNames.HeightProperty, 0);

            Property width = element[KnownNames.WidthProperty];
            Property height = element[KnownNames.HeightProperty];

            if (Kind == SizeGripKind.Width || Kind == SizeGripKind.WidthHeight)
            {
                width.Value = new Constant(x + dx);
            }

            if (Kind == SizeGripKind.Height || Kind == SizeGripKind.WidthHeight)
            {
                height.Value = new Constant(y + dy);
            }

            View.Redraw();
        }

        public override void Finish(MouseEventArgs e)
        {
            base.Finish(e);

            View.ActionTarget.PerformAction(
                new MultiAction(
                    Sized.Count > 1 ? string.Format("Změna velikosti {0:N0} elementů", Sized.Count) : ("Změna velikosti elementu " + Sized.Single()),
                    Sized.SelectMany(
                        (element, i) =>
                        {
                            Property width = element[KnownNames.WidthProperty];
                            Property height = element[KnownNames.HeightProperty];

                            var widthChangeAction = new ChangePropertyValueAction(width, width.Value, this.originalWidth[i]);
                            var heightChangeAction = new ChangePropertyValueAction(height, height.Value, this.originalHeight[i]);

                            var sizeActions = new List<ChangePropertyValueAction>();
                            if (Kind == SizeGripKind.Width || Kind == SizeGripKind.WidthHeight)
                            {
                                sizeActions.Add(widthChangeAction);
                            }
                            if (Kind == SizeGripKind.Height || Kind == SizeGripKind.WidthHeight)
                            {
                                sizeActions.Add(heightChangeAction);
                            }

                            return sizeActions;
                        })));
        }
    }
}
