﻿namespace Pikta.Editor
{
    using System;
    using System.Collections.Generic;
    using System.Drawing;
    using System.Linq;
    using System.Text;

    public class Stack : ContainerVisual
    {
        public Stack(IVisualProvider provider)
            : base(KnownNames.Stack, provider)
        {
        }

        public override void DrawElement(Graphics g, Element e, Rectangle container)
        {
            int sirka = GetWidth(e);
            int vyska = GetHeight(e);

            var r = new Rectangle(container.X, container.Y, sirka, vyska);

            OnRectangleChanged(e, r);

            using (var stetec = GetFill(e))
            {
                g.FillRectangle(stetec, r);
                //LastRectangles[element] = new Rectangle(rect.X, rect.Y, sirka, vyska);
            }

            // okraj
            using (var p = GetStroke(e))
            {
                g.DrawRectangle(p, r);
            }

            using (var vnitrek = new Region(r))
            {
                var state = ClipToRegion(g, vnitrek);

                int x = container.X;
                int y = container.Y;
                int vyska_radku = 0;
                int sirka_sloupce = 0;
                bool prvni = true;

                bool isHorizontal = GetOrientation(e) == Orientation.Horizontal;

                foreach (Element potomek in GetContent(e))
                {
                    int pw = GetWidth(potomek);
                    int ph = GetHeight(potomek);
                    int okraj = 0; // potomek.GetInt("margin");

                    pw += 2 * okraj;
                    ph += 2 * okraj;

                    if (isHorizontal)
                    {
                        // pokud se ma zalomit,
                        if (x + pw > r.Right)
                        {
                            // pokud neni prvni, zalomi se
                            if (!prvni)
                            {
                                // zalomeni
                                x = container.X;
                                y += vyska_radku;
                                vyska_radku = 0;
                            }
                        }
                    }
                    else
                    {
                        // pokud se ma zalomit,
                        if (y + ph > r.Bottom)
                        {
                            // pokud neni prvni, zalomi se
                            if (!prvni)
                            {
                                // zalomeni
                                y = container.Y;
                                x += sirka_sloupce;
                                sirka_sloupce = 0;
                            }
                        }
                    }

                    prvni = false;

                    vyska_radku = Math.Max(vyska_radku, ph);
                    sirka_sloupce = Math.Max(sirka_sloupce, pw);

                    GetHandler(potomek).DrawElement(g, potomek, new Rectangle(x + okraj, y + okraj, pw, ph));

                    if (isHorizontal)
                    {
                        x += pw;
                    }
                    else
                    {
                        y += ph;
                    }
                }

                g.Restore(state);
            }
        }

        private Orientation GetOrientation(Element e)
        {
            var vertical = Namespace.Global["Vertikální"];
            var el = e.GetElement("orientace", vertical);

            return el == vertical ? Orientation.Vertical : Orientation.Horizontal;
        }

        private enum Orientation
        {
            Vertical,
            Horizontal
        }
    }
}
