﻿using System;
using System.Collections.Generic;
using System.Drawing;
using System.Linq;
using System.Text;

namespace Pikta.Editor
{
    public class Text : Shape
    {
        private static readonly Bitmap helperBitmap = new Bitmap(10, 10);
        private static readonly Graphics helperGraphics;

        static Text()
        {
            helperGraphics = Graphics.FromImage(helperBitmap);
        }

        public Text()
            : base("Text")
        {
        }

        public override void DrawElement(Graphics g, Element e, Rectangle container)
        {
            // ziskame parametry
            var sirka = GetWidth(e);
            var vyska = GetHeight(e);

            var plocha = new Rectangle(container.Location, new Size(sirka, vyska));

            OnRectangleChanged(e, plocha);

            // vypln
            using (var s = GetFill(e))
            using (var font = CreateFont(GetTextSize(e)))
            {
                g.DrawString(GetText(e), font, s, plocha.Location);
            }

            //// okraj
            //using (var p = GetStroke(e))
            //{
            //    g.DrawEllipse(p, plocha);
            //}
        }

        public static int GetTextWidth(string text, double size)
        {
            return MeasureText(text, size).Width;
        }

        public static int GetTextHeight(string text, double size)
        {
            return MeasureText(text, size).Height;
        }

        protected string GetText(Element e)
        {
            return e.GetString("Text");
        }

        private float GetTextSize(Element e)
        {
            return (float)e.GetNumber("velikostTextu");
        }

        private static Font CreateFont(float size)
        {
            return new Font(FontFamily.GenericSansSerif, size);
        }

        private static Size MeasureText(string text, double size)
        {
            using (var f = CreateFont((float)size))
            {
                SizeF s = helperGraphics.MeasureString(text, f);
                return new Size((int)Math.Ceiling(s.Width), (int)Math.Ceiling(s.Height));
            }
        }
    }
}
