﻿namespace Pikta.Editor
{
    using System;
    using System.Collections.Generic;
    using System.Linq;
    using System.Text;
    using System.Diagnostics.Contracts;
    using System.Drawing;
    using System.Reactive.Subjects;

    /// <summary>
    /// Reprezentuje třídu, která obsluhuje vzhled a interakci potomků určitého protypu.
    /// </summary>
	public abstract class Visual
	{
        private readonly Subject<Tuple<Element, Rectangle>> rectangleChanged = new Subject<Tuple<Element, Rectangle>>();

		protected Visual(string prototypeName)
		{
			Contract.Requires(prototypeName != null);

			Prototype = prototypeName;
		}

		/// <summary>
		/// Vrátí prototyp, jehož potomky tato třída obsluhuje.
		/// </summary>
		public string Prototype { get; private set; }

        public IObservable<Tuple<Element, Rectangle>> RectangleChanged { get { return this.rectangleChanged; } }

		/// <summary>
		/// Vykreslí zadaný element do vymezeného prostoru.
		/// </summary>
		public abstract void DrawElement(Graphics g, Element e, Rectangle container);

        protected void OnRectangleChanged(Element e, Rectangle r)
        {
            this.rectangleChanged.OnNext(Tuple.Create(e, r));
        }
	}
}
