﻿namespace Pikta.Tests
{
    using System;
    using System.IO;
    using Microsoft.VisualStudio.TestTools.UnitTesting;
    using Pikta.Expressions;

    [TestClass]
    public class ElementSerializerTest
    {
        private readonly ElementSerializer serializer = new ElementSerializer(Namespace.Global);

        private const string RootName = "Kořen";

        [TestInitialize]
        public void TestInitialize()
        {
            Namespace.Global.Clear();
            serializer.Load("Colors.pikta");
            serializer.Load("Shapes.pikta");
            serializer.Load("Containers.pikta");
        }

        [TestMethod]
        public void LoadCloneBeforePrototype()
        {
            serializer.Load("KlonPrototyp.pikta");

            Assert.IsTrue(Namespace.Global.Contains("Klon"), "Nenačetl se klon.");
            Assert.IsTrue(Namespace.Global.Contains("Prototyp"), "Nenačetl se prototyp.");
        }

        [TestMethod]
        public void DoubleElementConstantReference()
        {
            var root = Element.Base.Clone();
            Namespace.Global[RootName] = root;

            var noName = Element.Base.Clone();
            root["a"].Value = new Constant(noName);
            root["b"].Value = new Constant(noName);

            string tempFile = Path.GetTempFileName();

            try
            {
                // save and reload
                serializer.Save(root, tempFile);
                Namespace.Global.Clear();
                serializer.Load(tempFile);

                root = Namespace.Global[RootName];
                var noNameA = (Element)((Constant)root["a"].Value).Value;
                var noNameB = (Element)((Constant)root["b"].Value).Value;

                Assert.IsTrue(noNameA == noNameB);
            }
            finally
            {
                File.Delete(tempFile);
            }
        }

        [TestMethod]
        public void UnnamedPrototypePrototype()
        {
            var root = Element.Base.Clone();
            Namespace.Global[RootName] = root;

            var noName = Element.Base.Clone();
            var potomek = noName.Clone();
            Namespace.Global["potomek"] = potomek;

            root["p"].Value = new Constant(potomek);

            string tempFile = Path.GetTempFileName();

            try
            {
                // save and reload
                serializer.Save(root, tempFile);
                Namespace.Global.Clear();
                serializer.Load(tempFile);

                root = Namespace.Global[RootName];
                potomek = (Element)((Constant)root["p"].Value).Value;

                Assert.AreSame(Element.Base, potomek.Prototype.Prototype);
            }
            finally
            {
                File.Delete(tempFile);
            }
        }
    }
}
