﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Pikta.Expressions;

namespace Pikta
{
	public static class Converters
	{
		public static bool CanConvertTo(Type source, PropertyType target)
		{
            if (target is NumberType)
            {
                return source == typeof(double) || source == typeof(int) || source == typeof(string);
            }
            else if (target is TextType)
            {
                return source == typeof(double) || source == typeof(int) || source == typeof(string);
            }
            else if (target is ElementType)
            {
                return source == typeof(Element);
            }

            return false;
        }

		public static bool CanConvertTo(PropertyType source, PropertyType target)
		{
            if (target is NumberType)
            {
                return source is NumberType || source is TextType;
            }
            else if (target is TextType)
            {
                return source is NumberType || source is TextType;
            }
            else if (target is ElementType)
            {
                return source is ElementType;
            }

            return false;
		}

		public static double? ToNumberOrNull(Result result)
		{
			if(result.Value is double)
                return (double)result.Value;
            else if (result.Value is int)
                return (int)result.Value;
            else if (result.Value is string)
			{
                string str = (string)result.Value;
				double cislo;
				if(double.TryParse(str, out cislo))
					return cislo;
			}

			return null;
		}

		public static double ToNumber(Result result)
		{
			double? cislo = ToNumberOrNull(result);
			if(cislo.HasValue)
				return cislo.Value;
			else
				throw new ArgumentException(string.Format("Hodnotu {0} nelze vzít jako číslo.", result));
		}

		public static string ToTextOrNull(Result result)
		{
            if (result.Value is double)
                return result.Value.ToString();
            else if (result.Value is int)
                return result.ToString();
            else if (result.Value is string)
                return (string)result.Value;
            else
                return null;
		}

        public static string ToText(Result result)
        {
            string text = ToTextOrNull(result);
            if (text != null)
            {
                return text;
            }
            else
            {
                throw new ArgumentException(string.Format("Hodnotu {0} nelze vzít jako text.", result));
            }
        }


        public static Element AsElement(Result result)
        {
            return result.Value as Element;
        }

		public static Element ToElement(Result result)
		{
            var e = AsElement(result);
			if(e != null)
				return e;
			throw new ArgumentException(string.Format("Hodnotu {0} nelze vzít jako element.", result));
		}
    }
}
