﻿using System;
using System.Collections.Generic;
using System.Diagnostics.Contracts;
using System.Linq;
using System.Text;
using Pikta.Expressions;

namespace Pikta
{
	public static class ElementExtensions
	{
		/// <summary>
		/// Vrátí výčet předků elementu (počínaje prototypem).
		/// </summary>
		public static IEnumerable<Element> GetAncestors(this Element element)
		{
			var prvek = element.Prototype;
			while(prvek != null)
			{
				yield return prvek;
				prvek = prvek.Prototype;
			}
		}

        public static bool IsDescendantOf(this Element element, Element ancestor)
        {
            return element.GetAncestors().Contains(ancestor);
        }

        public static bool IsDescendantOf(this Element element, string ancestorName)
        {
            return element.GetAncestors().Any(a => NameComparer.Instance.Equals(a.GetName(), ancestorName));
        }

		/// <summary>
		/// Vyhodnotí výraz a vrátí jeho hodnotu.
		/// </summary>
		/// <param name="context">Element, v jehož kontextu se má výraz vyhodnotit.</param>
		/// <param name="expression">Výraz, který se má vyhodnotit.</param>
		public static Result Evaluate(this Element context, Expression expression)
		{
			return Evaluation.Evaluate(expression, context);
		}

		public static Result Evaluate(this Element context, string propertyName)
		{
			return Evaluation.Evaluate(context[propertyName].Value, context);
		}

        [Pure]
        public static string GetName(this Element element)
		{
			return Namespace.Global.GetName(element);
		}
	}
}
