﻿using System;
using System.Collections.Generic;
using System.Diagnostics.Contracts;
using System.Linq;
using System.Text;

namespace Pikta
{
    public sealed class ElementType : PropertyType, IEquatable<ElementType>
    {
        private static ElementType any = new ElementType(null);

        private ElementType(string baseElementName)
        {
            this.BaseElementName = baseElementName;
        }

        /// <summary>
        /// Gets a type that contains any element.
        /// </summary>
        public static new ElementType Any { get { return any; } }

        public string BaseElementName { get; private set; }

        /// <summary>
        /// Returns a type that contains elements based on specified element only.
        /// </summary>
        /// <param name="baseElementName">The base element name.</param>
        public static ElementType GetTypeBasedOn(string baseElementName)
        {
            Contract.Requires(!string.IsNullOrEmpty(baseElementName));
            return new ElementType(baseElementName);
        }

        public bool Equals(ElementType other)
        {
            if (other == null) return false;

            return NameComparer.Instance.Equals(this.BaseElementName, other.BaseElementName);
        }
    }
}
