﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace Pikta.Expressions
{
	public abstract class BinaryExpression : Expression
	{
		protected BinaryExpression(Expression left, Expression right)
		{
			Left = left;
			Right = right;
		}

		public Expression Left { get; private set; }
		public Expression Right { get; private set; }

		public override IEnumerable<SymbolExpression> AllSymbolExpressions
        {
            get { return Left.AllSymbolExpressions.Union(Right.AllSymbolExpressions); }
        }

        public override IEnumerable<Constant> AllConstants
        {
            get { return Left.AllConstants.Union(Right.AllConstants); }
        }

		public bool Equals(BinaryExpression other)
		{
			return other != null && other.Left.Equals(Left) && other.Right.Equals(Right);
		}

		public override bool Equals(Expression other)
		{
			return Equals(other as BinaryExpression);
		}

		public override bool Equals(object obj)
		{
			return Equals(obj as BinaryExpression);
		}

		public override int GetHashCode()
		{
			return Left.GetHashCode() ^ Right.GetHashCode();
		}
	}
}
