﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace Pikta.Expressions
{
    public static class CommonFunctions
    {
        public static void Define()
        {
            // scene
            Add("TentoElement", 0, (c, args) => Result.Success(c));

            // math
            Add("Pi", 0, (c, args) => Result.Success(Math.PI));
            Add("Abs", 1, CreateNumber1(Math.Abs));
            Add("Cos", 1, CreateNumber1(Math.Cos));
            Add("Sin", 1, CreateNumber1(Math.Sin));
            Add("Celé", 1, CreateNumber1(n => (int)n));
            Add("Mocnina", 2, CreateNumber2((x, p) => Math.Pow(x, p)));
            Add("Odmocnina", 1, CreateNumber1(Math.Sqrt));

            // time
            Add("Hodina", 0, (_, __) => Result.Success(DateTime.Now.Hour));
            Add("Minuta", 0, (_, __) => Result.Success(DateTime.Now.Minute));
            Add("Sekunda", 0, (_, __) => Result.Success(DateTime.Now.Second));

            // bool
            Add("Pokud", 3, (c, args) => IsTrue(args.ElementAt(0)) ? args.ElementAt(1) : args.ElementAt(2));
        }

        private static void Add(string name, int arity, Evaluation.FunctionBody body)
        {
            Evaluation.FunctionsLibrary[new FunctionDefinition(name, arity)] = body;
        }

        private static Evaluation.FunctionBody CreateNumber1(Func<double, double> body)
        {
            return (c, args) => Result.Success(body(Converters.ToNumber(args.Single())));
        }

        private static Evaluation.FunctionBody CreateNumber2(Func<double, double, double> body)
        {
            return (c, args) => Result.Success(body(Converters.ToNumber(args.ElementAt(0)), Converters.ToNumber(args.ElementAt(1))));
        }

        private static bool IsTrue(Result r)
        {
            // Zero is false, everything else is true
            double? number = Converters.ToNumberOrNull(r);
            return number == null || number.Value != 0;
        }
    }
}
