﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Diagnostics.Contracts;
using System.Globalization;

namespace Pikta.Expressions
{
	public class Constant : Expression
	{
		public Constant(double value)
		{
			Value = value;
		}

		public Constant(string value)
		{
			Contract.Requires(value != null);
			Value = value;
		}

		public Constant(Element value)
		{
			Contract.Requires(value != null);
			Value = value;
		}

		public object Value { get; private set; }

		public override IEnumerable<SymbolExpression> AllSymbolExpressions
		{
			get { return Enumerable.Empty<SymbolExpression>(); }
		}

        public override IEnumerable<Constant> AllConstants
        {
            get { return new[] { this }; }
        }

		public bool Equals(Constant other)
		{
			return other != null && other.Value.Equals(Value);
		}

		public override bool Equals(Expression other)
		{
			return Equals(other as Constant);
		}

		public override bool Equals(object obj)
		{
			return Equals(obj as Constant);
		}

		public override int GetHashCode()
		{
			return Value.GetHashCode();
		}

		public override string ToString()
		{
            return ToString(CultureInfo.CurrentCulture);
		}

        public override string ToString(IFormatProvider formatProvider)
        {
            if (Value is string)
            {
                return string.Format(formatProvider, "\"{0}\"", Value);
            }
            else if(Value is double)
            {
                return ((double)Value).ToString(formatProvider);
            }
            else
            {
                return Value.ToString();
            }
        }
	}
}
