﻿using System;
using System.Collections.Generic;
using System.Diagnostics.Contracts;
using System.Globalization;
using System.Linq;
using System.Text;

namespace Pikta.Expressions
{
    public class FunctionCall : Expression
    {
        public FunctionCall(FunctionDefinition definition, IEnumerable<Expression> args)
        {
            Contract.Requires(definition != null);
            Contract.Requires(args != null);

            Definition = definition;
            Arguments = args.ToArray();
        }

        public FunctionDefinition Definition { get; private set; }

        public IList<Expression> Arguments { get; private set; }

        public override IEnumerable<SymbolExpression> AllSymbolExpressions
        {
            get { return Arguments.SelectMany(e => e.AllSymbolExpressions); }
        }

        public override IEnumerable<Constant> AllConstants
        {
            get { return Arguments.SelectMany(e => e.AllConstants); }
        }

        public override bool Equals(Expression other)
        {
            var fc = other as FunctionCall;

            if (fc == null)
            {
                return false;
            }

            return this.Definition.Equals(fc.Definition) && this.Arguments.SequenceEqual(fc.Arguments);
        }

        public override bool Equals(object obj)
        {
            return Equals(obj as FunctionCall);
        }

        public override int GetHashCode()
        {
            return this.Arguments.Aggregate(
                this.Definition.GetHashCode(),
                (hash, e) => hash ^ e.GetHashCode());
        }

        public override string ToString()
        {
            return ToString(CultureInfo.CurrentCulture);
        }

        public override string ToString(IFormatProvider formatProvider)
        {
            return string.Format(formatProvider, "{0}({1})", Definition.Name, string.Join("; ", this.Arguments.Select(e => e.ToString(formatProvider))));
        }
    }
}
