﻿using System;
using System.Collections.Generic;
using System.Diagnostics.Contracts;
using System.Linq;
using System.Text;

namespace Pikta.Expressions
{
    public class FunctionDefinition : IEquatable<FunctionDefinition>
    {
        public FunctionDefinition(string name, int arity)
        {
            Contract.Requires(name != null);
            Contract.Requires(Parser.IsValidElementName(name));
            Contract.Requires(arity >= 0);

            Name = name;
            Arity = arity;
        }

        public string Name { get; private set; }

        public int Arity { get; private set; }

        public bool Equals(FunctionDefinition other)
        {
            if (other == null) return false;

            return NameComparer.Instance.Equals(this.Name, other.Name) && this.Arity == other.Arity;
        }

        public override bool Equals(object obj)
        {
            return Equals(obj as FunctionDefinition);
        }

        public override int GetHashCode()
        {
            return NameComparer.Instance.GetHashCode(this.Name) ^ this.Arity;
        }

        public override string ToString()
        {
            return string.Format("{0}/{1}", this.Name, this.Arity);
        }
    }
}
