﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Diagnostics.Contracts;

namespace Pikta.Expressions
{
	/// <summary>
	/// Vypočítaná hodnota výrazu.
	/// </summary>
	public struct Result
	{
		private Result(object value, PropertyType type)
			: this()
		{
			Contract.Requires(value != null);
            Contract.Requires(type != null);
			Value = value;
            ValueType = type;
		}

		private Result(object defaultValue, string errorMessage)
			: this()
		{
			Contract.Requires(defaultValue != null);
			Value = defaultValue;
            ValueType = PropertyType.Any;
            ErrorMessage = errorMessage;
		}

		public object Value { get; private set; }

        public PropertyType ValueType { get; private set; }

		public string ErrorMessage { get; private set; }

		public bool HasError { get { return !string.IsNullOrEmpty(ErrorMessage); } }

		public override string ToString()
		{
			if(HasError)
			{
				return string.Format("({0})", ErrorMessage);
			}
			else
			{
				return Value.ToString();
			}
		}

		public static Result Success(double value)
		{
			return new Result(value, NumberType.Instance);
		}

		public static Result Success(string value)
		{
			return new Result(value, TextType.Instance);
		}

		public static Result Success(Element value)
		{
			return new Result(value, ElementType.Any);
		}

		public static Result Error(string message)
		{
			return new Result(0, message);
		}
	}
}
