﻿namespace Pikta
{
    using System;
    using System.Collections.Generic;
    using System.Diagnostics.Contracts;
    using System.Linq;
    using System.Text;

    /// <summary>
	/// Přiřazuje elementům názvy.
	/// </summary>
	public class Namespace
	{
		private static Namespace global = new Namespace();

		static Namespace()
		{
            Global.Clear();
		}

		public Namespace()
		{
			Mappings = new Dictionary<string, Element>(NameComparer.Instance);
		}

		public static Namespace Global { get { return global; } }

        public IEnumerable<Element> Elements { get { return this.Mappings.Values; } }

        private IDictionary<string, Element> Mappings { get; set; }

		public Element this[string name]
		{
			get
			{
				return Mappings[name];
			}
			set
			{
				Contract.Requires(name != null);
				Contract.Requires(value != null);
				Contract.Requires(!Contains(name), "The element with specified name already exists in the namespace.");

				Mappings[name] = value;
			}
		}

		[Pure]
		public bool Contains(string name)
		{
			return Mappings.ContainsKey(name);
		}

		public bool TryGetElement(string name, out Element element)
		{
			return Mappings.TryGetValue(name, out element);
		}

		/// <summary>
		/// Vrátí název zadaného elementu v tomto jmenném prostoru, nebo null.
		/// </summary>
		/// <param name="element">Element, který se má vyhledat.</param>
        [Pure]
		public string GetName(Element element)
		{
			Contract.Requires(element != null);

			var par = Mappings.FirstOrDefault(p => p.Value == element);
			
			if(par.Key != null)
			{
				return par.Key;
			}
			else
			{
				return null;
			}
		}

		public bool Rename(string currentName, string newName)
		{
			Element element;

			if(Mappings.TryGetValue(currentName, out element) && !Contains(newName))
			{
				Mappings.Remove(currentName);
				Mappings.Add(newName, element);
				return true;
			}
			else
			{
				return false;
			}
		}

        public string AddWithUniqueName(Element element)
        {
            var uniqueName = CreateUniqueName(element);
            this[uniqueName] = element;
            return uniqueName;
        }

        public void Remove(Element element)
        {
            foreach (var mapping in Mappings)
            {
                if (mapping.Value == element)
                {
                    Mappings.Remove(mapping.Key);
                    return;
                }
            }
        }

        public void Clear()
        {
            Mappings.Clear();
            this["Báze"] = Element.Base;
        }

        private string CreateUniqueName(Element element)
        {
            int cislo = 1;
            string nazev;

            string prototypName = element.Prototype.GetName() ?? "g";

            do
            {
                nazev = string.Format("{0}{1}", prototypName, cislo++);
            }
            while (this.Contains(nazev));

            return nazev;
        }
    }
}
