﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Pikta.Expressions;
using System.Diagnostics.Contracts;

namespace Pikta
{
	/// <summary>
	/// Záznam definující přímou (nezděděnou) vlastnost elementu.
	/// </summary>
	public class PropertyDefinition : IComparable<PropertyDefinition>
	{
		private Expression expression;

		internal PropertyDefinition(Element element, string name, PropertyType type)
		{
			Contract.Requires(element != null);
			Contract.Requires(name != null);

			Element = element;
			Name = name;
			Type = type;
		}

		/// <summary>Vrátí element, který tuto vlastnost definuje.</summary>
		public Element Element { get; private set; }

		/// <summary>Vrátí název vlastnosti.</summary>
		public string Name { get; private set; }

        // TODO: Je to k něčemu?
		public PropertyType Type { get; private set; }

		//public bool Private { get; set; }
		//public bool ReadOnly { get; set; }

		/// <summary>
		/// Vrátí/nastaví hodnotu vlastnosti.
		/// </summary>
		public Expression Value
		{
			get { return expression; }
			set
			{
				Contract.Requires(value != null);

				expression = value;
			}
		}

		private Exception GetPropertyReadOnlyException()
		{
			return new InvalidOperationException(string.Format("Vlastnost '{0}' objektu '{1}' je určena pouze pro čtení.",
				Name, Element));
		}

		public override string ToString()
		{
			if(expression != null)
				return expression.ToString();
			else
				return "null";
		}

		public int CompareTo(PropertyDefinition other)
		{
			if(other == null)
				return 1;
			else
			{
				return NameComparer.Instance.Compare(Name, other.Name);
			}
		}
	}
}
